/*	libiac - Internal Application Communication Library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#ifndef __IAC_H
#define __IAC_H

#define	DBUS_API_SUBJECT_TO_CHANGE

#include <glib.h>

/**
 * This structure holds Inter-App Communication
 * context for the current appliaction.
 */
typedef	struct _iac_context_t 	iac_context_t;

/**
 * This structure is used as values container for
 * arguments and returns of most IAC functions.
 */
typedef	struct _iac_arguments_t	iac_arguments_t;

/**
 * Value type definition of iac_arguments_t 
 */
#define IAC_TYPE_INVALID				((int) '\0')
#define IAC_TYPE_INT32					((int) 'i')
#define IAC_TYPE_UINT32					((int) 'u')
#define IAC_TYPE_DOUBLE					((int) 'd')
#define IAC_TYPE_BOOLEAN				((int) 'b')
#define IAC_TYPE_STRING					((int) 's')

/**
 * This structure stores one value element of
 * iac_argument_t.
 */
typedef struct {
  int		vtype;				/**<type of value, leads with IAC_TYPE_ */
  union {
    guint32		 u32;
    gint32		 i32;
    gboolean	 b;
    gdouble		 d;
    gchar			*s;
  } value;						/**<real content of value */
} iac_value_t;

/**
 * This functione create a new iac_argument_t
 * instance. 
 * 
 * @return	A newly-created iac_arguments_t instance, and
 * should be freed by calling iac_arguments_free() when it
 * is no longer needed.
 */
iac_arguments_t *		iac_arguments_new									(void);

/**
 * This functione create a new iac_argument_t
 * instance with and then appends a set of argument
 * values to it. It is a shortcut for iac_arguments_new() 
 * and iac_arguments_appends_args().
 * 
 * @param	argument_type	The type of the first argument.
 * @param	...						The first argument value, and a 
 * <type, value> pair list of other arguments. This list
 * must end wit a IAC_TYPE_INVALID type.
 * 
 * @return	A newly-created iac_arguments_t instance, and
 * should be freed by calling iac_arguments_free() when it
 * is no longer needed.
 */
iac_arguments_t *		iac_arguments_new_with_args				(int									 first_val_type,
																																						 ...);
																																						 
/**
 * This functions destroy and free the memory alloction
 * for a spefic iac_arguments_t instance.
 * 
 * @param	args	An iac_arguments_t instance.
 */
void								iac_arguments_free								(iac_arguments_t			*args);

/**
 * This function retrieve the number of values existed 
 * in specifci iac_arguments_t instance.
 * 
 * @param	args	An iac_arguments_t instance.

 * @return	number of values or 0 indicates a empty
 * iac_arguments_t instance.
 */
int									iac_arguments_size								(iac_arguments_t			*args);

/**
 * This function append a set of values to an iac_arguments_t
 * instance.
 * 
 * @param	args					An iac_arguments_t instance.
 * @param	argument_type	The type of the first argument.
 * @param	...						The first argument value, and a 
 * <type, value> pair list of other arguments. This list
 * must end wit a IAC_TYPE_INVALID type.
 */
void								iac_arguments_append_args	 				(iac_arguments_t			*args,
																											 int									 first_val_type,
																											 											 ...);
/**
 * This function retrieve one value in an iac_arguments_t
 * instance at index position.
 * 
 * @param	args		A iac_arguments_t instance.
 * @param index		Index of the value to retrieve.
 * 
 * @return	A const pointer to the value or NULL indicates
 * an invalid index or other errors occurred.s
 */
const iac_value_t *	iac_arguments_get_arg							(iac_arguments_t			*args,
																											 int									 index);

/**
 * This functions retrieve a set of values in an iac_arguments_t
 * instance. The valued
 * 
 */
void								iac_arguments_get_args						(iac_arguments_t			*args,
																											 int									 first_val_type,
																											 											 ...);
void								iac_arguments_get_args_start_at		(iac_arguments_t			*args,
																											 int									 index,
																											 int									 first_val_type,
																											 											 ...);

typedef enum {
	IAC_OK = 0,
	IAC_ERROR = -1,
} iac_return_t;

typedef	enum
{
	IAC_METHOD = 0,
	IAC_SIGNAL,	
} iac_listener_type_t;

typedef iac_return_t	(*iac_listener_func)						(const gchar					*interface,
																											 const gchar					*method,
																											 iac_arguments_t			*args,
																											 iac_arguments_t			*retval,
																											 gpointer							 user_data);
																											 
typedef void					(*iac_async_callback)						(const gchar					*interface,
																											 const gchar					*method,
																											 iac_arguments_t			*retval,
																											 gpointer							 user_data);

iac_context_t *			iac_initialize										(void);
void								iac_deinitialize									(iac_context_t				*context);

iac_return_t				iac_register_service							(iac_context_t				*context,
																											 const gchar					*serivce);
iac_return_t				iac_register_listener							(iac_context_t				*context,
																											 const gchar					*object,
																											 const gchar					*interface,
																											 iac_listener_func		 callback,
																											 gpointer							 user_data);
iac_return_t				iac_unregister_listener 					(iac_context_t				*context,
																											 const gchar					*interface,
																											 const gchar					*object);

iac_return_t				iac_start_service									(iac_context_t				*context,
																											 const gchar					*service,
																											 const gchar					*method);
																											 
iac_return_t				iac_execute_service								(iac_context_t				*context,
																											 const gchar					*service,
																											 const gchar					*object,
																											 const gchar					*interface,
																											 const gchar					*method,
																											 iac_arguments_t			*args,
																											 iac_arguments_t			*retval);
																											 
iac_return_t				iac_execute_service_async					(iac_context_t				*context,
																											 const gchar					*service,
																											 const gchar					*object,
																											 const gchar					*interface,
																											 const gchar					*method,
																											 iac_arguments_t			*args,
																											 iac_async_callback		 async_cb,
																											 void									*user_data);

iac_return_t				iac_emit_signal										(iac_context_t				*context,
																											 const gchar					*object,
																											 const gchar					*interface,
																											 const gchar					*method,
																											 iac_arguments_t			*args);

#endif /*IAC_H_*/
