/*	libiac - Internal Application Communication Library
 *
 *	Authors: YE Nan <nan.ye@orange-ftgroup.com> 
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libintl.h>

#include "iac.h"
#include "iac-internal.h"

#define _(x) gettext(x)

static void
iac_value_free (iac_value_t * val)
{
  if (val->vtype == IAC_TYPE_STRING)
  {
		g_free (val->value.s);
  }
  
  val->vtype = IAC_TYPE_INVALID;
  
  return;
}

void
iac_arguments_print (iac_arguments_t * args)
{
	GArray * arguments = NULL;
	gint i;
	if (args == NULL)
	{
		return;
	}
	
	g_print("%s(): IAC Arguments 0x%08x\n", __FUNCTION__, (guint32)args);
	arguments = args->values;
	if (arguments == NULL)
	{
		return;
	}
	
	g_print("%s(): Count: %d\n", __FUNCTION__, arguments->len);
	for (i = 0; i < arguments->len; i++)
	{
		iac_value_t * val = &g_array_index(arguments, iac_value_t, i);
		
		g_print("%s(): %c- ",
						__FUNCTION__,
						(i == arguments->len - 1) ? '`' : '|');
						
		switch (val->vtype) {
			case IAC_TYPE_INT32:
				g_print("type = int32  , value = %d\n", val->value.i32);
				break;
				
			case IAC_TYPE_UINT32:
				g_print("type = uint32 , value = %u\n", val->value.u32);
				break;
			
			case IAC_TYPE_BOOLEAN:
				g_print("type = boolean, value = %s\n", (val->value.b) ? "true" : "false");
				break;
	  
			case IAC_TYPE_DOUBLE:
				g_print("type = double , value = %f\n", val->value.d);
				break;
			
			case IAC_TYPE_STRING:
				g_print("type = string , value = %s\n", val->value.s);
				break;
			
			default:
				g_print("type = unknow , value = 0\n");
				break;	
		}
	}
	
	return;
}

static void
iac_arguments_append_args_valist (iac_arguments_t	*args,
																	int							 first_val_type,
																	va_list					 var_args)
{
	int type;
	
	if (args == NULL)
	{
		return;
	}
	
	type = first_val_type;
	while (type != IAC_TYPE_INVALID)
	{
		iac_value_t val;
		
		val.vtype = type;
		switch (type) {
			case IAC_TYPE_INT32:
				val.value.i32 = va_arg(var_args, gint32);
				break;
				
			case IAC_TYPE_UINT32:
				val.value.u32 = va_arg(var_args, guint32);
				break;
			
			case IAC_TYPE_BOOLEAN:
				val.value.b = va_arg(var_args, gboolean);
				break;
	  
			case IAC_TYPE_DOUBLE:
				val.value.d = va_arg(var_args, gdouble);
				break;
			
			case IAC_TYPE_STRING:
			{
				gchar * str = va_arg(var_args, gchar *);
				if (str != NULL)
				{
					val.value.s = g_strdup(str);
				}				
				else
				{
					val.vtype = IAC_TYPE_INVALID;
					val.value.i32 = 0;
				}
				break;
			}
			
			default:
				val.vtype = IAC_TYPE_INVALID;
				val.value.i32 = 0;	    
				break;	
		}
		
		g_array_append_val(args->values, val);
		
		type = va_arg(var_args, int);
	}
	
	return;
}

iac_arguments_t *
iac_arguments_new (void)
{
	iac_arguments_t * args = NULL;
	
	args = g_new0(iac_arguments_t, 1);
	if (args == NULL)
	{
		return NULL;
	}
	
	args->values = g_array_new(FALSE, 
														 FALSE,
														 sizeof(iac_value_t));
										 
	return args;
}

iac_arguments_t *
iac_arguments_new_with_args (int	first_val_type,
																	...)
{
	iac_arguments_t * args = NULL;
	va_list var_args;
	
	args = iac_arguments_new();
	if (args == NULL)
	{
		return NULL;
	}

	va_start(var_args, first_val_type);
	iac_arguments_append_args_valist(args,
																	 first_val_type,
																	 var_args);
	va_end(var_args);
	
	return args;
}

void
iac_arguments_free (iac_arguments_t * args)
{
	if (args == NULL)
	{
		return;
	}
	
	if (args->values)
	{
		gint i;
	
		for (i = 0; i < args->values->len; i++)
		{
			iac_value_free(&g_array_index(args->values, iac_value_t, i));
		}
		
    g_array_free(args->values, TRUE);
	}
	
	g_free(args);
	
	return;
}

void
iac_arguments_append_args (iac_arguments_t	*args,
													 int							 first_val_type,
																						 ...)
{
	va_list var_args;
	
	va_start(var_args, first_val_type);
	
	iac_arguments_append_args_valist(args,
																	 first_val_type,
																	 var_args);

	va_end(var_args);
	
	return;
}

static void
iac_arguments_get_args_start_at_valist (iac_arguments_t		*args,
																				int								 index,
																				int								 first_val_type,
																				va_list						 var_args)

{
	int type;
	gint i;
	
	if (args == NULL)
	{
		return;
	}
	
	type = first_val_type;
	i = index;
	while (type != IAC_TYPE_INVALID)
	{
		gpointer * dest = va_arg(var_args, gpointer);
		
		if (i < 0 || i >= args->values->len)
		{
			switch(type)
			{
				case IAC_TYPE_INT32:
					*((gint32 *)dest) = 0;
					break;
					
				case IAC_TYPE_UINT32:
					*((guint32 *)dest) = 0;
					break;
					
				case IAC_TYPE_BOOLEAN:
					*((gboolean *)dest) = FALSE;
					break;
					
				case IAC_TYPE_DOUBLE:
					*((gdouble *) dest) = 0.0;
					break;
				
				case IAC_TYPE_STRING:
					*((gchar **)dest) = NULL;
					break;					
				
				default:
					*((gint32 *)dest) = 0;
					break;				
			}
		}
		else
		{
			iac_value_t * val = NULL;
			
			val = &g_array_index(args->values, iac_value_t, i);
			
			switch (type) {
				case IAC_TYPE_INT32:
					*((gint32 *)dest) = val->value.i32;
					break;
					
				case IAC_TYPE_UINT32:
					*((guint32 *)dest) = val->value.u32;
					break;
				
				case IAC_TYPE_BOOLEAN:
					*((gboolean *)dest) = val->value.b;
					break;
		  
				case IAC_TYPE_DOUBLE:
					*((gdouble *)dest) = val->value.d;
					break;
				
				case IAC_TYPE_STRING:
				{
					if (val->value.s != NULL)
					{
						*((gchar **)dest) = g_strdup(val->value.s);
					}
					else
					{
						*((gchar **)dest) = NULL;
					}
					break;
				}
				
				default:
					*((gint32 *)dest) = 0;
					break;				
			}
		}
		
		type = va_arg(var_args, int);
		i++;
	}
	
	return;
}

const iac_value_t *
iac_arguments_get_arg	(iac_arguments_t			*args,
											 int									 index)
{
	if (args == NULL)
	{
		return NULL;
	}
	
	if (index < 0 || index >= args->values->len)
	{
		return NULL;
	}
	
	return &g_array_index(args->values, iac_value_t, index);
}

void
iac_arguments_get_args (iac_arguments_t		*args,
												int								 first_val_type,
																					 ...)
{
	va_list var_args;
	
	va_start(var_args, first_val_type);
	
	iac_arguments_get_args_start_at_valist(args,
																				 0,
																				 first_val_type,
																				 var_args);

	va_end(var_args);
	
	return;
}

void
iac_arguments_get_args_start_at (iac_arguments_t		*args,
																 int								 index,
																 int								 first_val_type,
																										 ...)
{
	va_list var_args;
	
	va_start(var_args, first_val_type);
	
	iac_arguments_get_args_start_at_valist(args,
																				 index,
																				 first_val_type,
																				 var_args);

	va_end(var_args);
	
	return;
}

int
iac_arguments_size (iac_arguments_t			*args)
{
	if (args == NULL)
	{
		return 0;
	}
	
	return args->values->len;
}

/*
vi:ts=2:nowrap:ai:expandtab 
*/
